#!/usr/bin/env xpyv
#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------


import unittest
from multipath_conf import MultipathConf

class MultipathTest(unittest.TestCase):
    def test_ignoring_comments(self):
        config = """# TEST
    #TEST
f 34#C
foo { # XXX
    bar "foobar"  #YYY
    bar bar#ioo
}#"""
        obj = MultipathConf(config)
        self.assertEquals(obj.config, {'foo': [{'bar': ['"foobar"', 'bar']}], 'f': ['34']})

    def test_nested_sections(self):
        config = """section1 {
    section2 {
        key "value1" #FDSFDS
    }
    section2 {
        key "spaced value"
    }
    section2 {
        key value
    }
}
"""
        obj = MultipathConf(config)
        self.assertEquals(obj.config, {'section1': [{'section2': [{'key': ['"value1"']}, {'key': ['"spaced value"']}, {'key': ['value']}]}]})
    
    def test_flat_section_with_special_chars(self):
        config = """
section {
    key "^(|[ )!*"
    key2 value_with_no_quotes
    key3 453
}
"""
        obj = MultipathConf(config)
        self.assertEquals(obj.config, {'section': [{'key3': ['453'], 'key2': ['value_with_no_quotes'], 'key': ['"^(|[ )!*"']}]})

    def test_real_multipath_conf(self):
        config = """
# Linux Multipath I/O Configuration File
# Version 1.5rc4
# Generated by IBM-XIV Host Attachment at 2009-11-18 15:3:26
# Do not manually modify this file!

blacklist {
    device {
        vendor "IBM-ESXS" #FDSFDS
    }
    device {
        vendor "LSILOGIC"
    }
    device {
        vendor "ATA"
    }
    device {
        vendor "VMware"
    }
    devnode "^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"
    devnode "^hd[a-z]"
    devnode "^cciss!c[0-9]d[0-9]*"
}

devices {
    device {
        rr_min_io 15
        product "NEvTRA"
        vendor "XIV"
        path_checker tur
        path_grouping_policy multibus
        path_selector "round-robin 0"
        no_path_retry 5
        polling_interval 3
        failback 15
    }
    device {
        rr_min_io 15
        product "2800XIV"
        vendor "IBM"
        path_checker tur
        path_grouping_policy multibus
        path_selector "round-robin 0"
        no_path_retry 5
        polling_interval 3
        failback 15
    }
    device {
        rr_min_io 15
        path_checker tur
        product "2810XIV"
        no_path_retry 5
        vendor "IBM"
        polling_interval 3
        path_grouping_policy multibus
        path_selector "round-robin 0"
        failback 15
    }
    device {
        rr_min_io 15
        path_checker tur
        product "NEXTRA"
        no_path_retry 5
        vendor "XIV"
        polling_interval 3
        path_grouping_policy multibus
        path_selector "round-robin 0"
        failback 15
    }
}
defaults {
    user_friendly_names yes
}
"""
        obj = MultipathConf(config)
        self.assertEquals(obj.config, {'blacklist': [{'device': [{'vendor': ['"IBM-ESXS"']}, {'vendor': ['"LSILOGIC"']}, {'vendor': ['"ATA"']}, {'vendor': ['"VMware"']}], 'devnode': ['"^(ram|raw|loop|fd|md|dm-|sr|scd|st)[0-9]*"', '"^hd[a-z]"', '"^cciss!c[0-9]d[0-9]*"']}], 'defaults': [{'user_friendly_names': ['yes']}], 'devices': [{'device': [{'rr_min_io': ['15'], 'product': ['"NEvTRA"'], 'vendor': ['"XIV"'], 'path_checker': ['tur'], 'path_grouping_policy': ['multibus'], 'path_selector': ['"round-robin 0"'], 'no_path_retry': ['5'], 'polling_interval': ['3'], 'failback': ['15']}, {'rr_min_io': ['15'], 'product': ['"2800XIV"'], 'vendor': ['"IBM"'], 'path_checker': ['tur'], 'path_grouping_policy': ['multibus'], 'path_selector': ['"round-robin 0"'], 'no_path_retry': ['5'], 'polling_interval': ['3'], 'failback': ['15']}, {'rr_min_io': ['15'], 'product': ['"2810XIV"'], 'vendor': ['"IBM"'], 'path_checker': ['tur'], 'path_grouping_policy': ['multibus'], 'path_selector': ['"round-robin 0"'], 'no_path_retry': ['5'], 'polling_interval': ['3'], 'failback': ['15']}, {'rr_min_io': ['15'], 'product': ['"NEXTRA"'], 'vendor': ['"XIV"'], 'path_checker': ['tur'], 'path_grouping_policy': ['multibus'], 'path_selector': ['"round-robin 0"'], 'no_path_retry': ['5'], 'polling_interval': ['3'], 'failback': ['15']}]}]})


def main():
    suite = unittest.TestSuite( [ 
        MultipathTest("test_ignoring_comments"),
        MultipathTest("test_nested_sections"),
        MultipathTest("test_flat_section_with_special_chars"),
        MultipathTest("test_real_multipath_conf"),
    ])
    unittest.TextTestRunner().run(suite)

if __name__ == "__main__":
    main()
