#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   

import threading, traceback, sys

from exceptions import CollectorTimedOut, CollectionError, CollectionSkip

TIMEOUT = 10

class InterruptableThread(threading.Thread):
    def __init__(self, func, event, *args, **kwargs):
        threading.Thread.__init__(self)
        self.result = None
        self.func = func
        self.args = args
        self.event = event
        self.kwargs = kwargs
    
    def run(self):
        try:
            self.result = self.func(self.event, *self.args, **self.kwargs)
        except (CollectionSkip), e:
            self.result = CollectionSkip(e)
        except (CollectionError), e:
            self.result = CollectorTimedOut(e)
        except Exception, e:
            exception_type, exception_value, exception_traceback = sys.exc_info()
            self.result = CollectionError('\n' + ''.join(traceback.format_exception(\
                                          exception_type, exception_value,
                                          exception_traceback)))

class CollectorThread(object):
    def __init__(self, timeout=TIMEOUT):
        self.timeout = timeout
        self.event = threading.Event()
        self.event.clear()

    def run(self, collectable, run_func_name='_collect', *args, **kwargs):
        interruptable_thread = InterruptableThread(getattr(collectable, run_func_name), self.event, *args, **kwargs)
        interruptable_thread.start()
        interruptable_thread.join(self.timeout)
        if interruptable_thread.isAlive():
            if hasattr(collectable, "proc"):
                collectable.proc.kill()
            self.event.set()
            raise CollectorTimedOut('thread has timed out')
        else:
            if type(interruptable_thread.result) is CollectionSkip:
                raise interruptable_thread.result
            if type(interruptable_thread.result) is CollectorTimedOut:
                raise interruptable_thread.result
            if type(interruptable_thread.result) is CollectionError:
                raise interruptable_thread.result
            return interruptable_thread.result

    def thread(self, collectable):
        def callable(*args, **kwargs):
            return self.run(collectable, *args, **kwargs)
        callable.__name__ = collectable.collect.__name__
        return callable
