#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   

from os import access, R_OK, stat
from os.path import getmtime, isfile, join
from time import localtime, strftime

from interfaces import ICollectable
from exceptions import CollectionError, CollectionSkip
from collector_thread import CollectorThread

TIMEOUT = 120
READ_SIZE = 512

class SimpleFileCollectable(ICollectable):
    """
    Collect metadata and data files
    """
    def __init__(self, description, archive, file_to_collect, timeout=2*TIMEOUT):
        ICollectable.__init__(self, description)
        self.archive = archive
        self.file_to_collect = file_to_collect
        self.add_attribute('type', 'file')
        self.collector_thread = CollectorThread(timeout)
        self.collect = self.collector_thread.thread(self)
        self.timeout = timeout
   
    def _can_open_and_read_one_byte(self, filepath):
        try:
            fd = open(filepath, 'rb')
            fd.close()
        except IOError:
            return False
        return True
    
    def read_file(self, event):
        if not self.archive.test_path(self.file_to_collect, True, event):
            raise CollectionSkip
    
    def add_file_to_archive(self):
        self.archive.add(self.file_to_collect)

    def _collect(self, event=None):
        self.event = event
        read_thread = CollectorThread(self.timeout)
        read_thread.run(self, 'read_file')
        if self.event.is_set():
            return
        self.add_file_to_archive()
        self.add_attribute("filepath", self.file_to_collect)
        self.add_attribute("collect-timestamp", strftime("%m/%d/%Y %I:%M:%S %p", localtime()))
        self.add_attribute("last-modified-timestamp", \
                           strftime("%m/%d/%Y %I:%M:%S %p", localtime(getmtime(self.file_to_collect))))
