#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

from os import path, environ
from commands import getoutput
from subprocess import PIPE, Popen

from interfaces import IXIVDiag
from collector.collectable.files import SimpleFileCollectable as FileItem
from collector.collectable.commands import SimpleCommandCollectable as CommandItem
from collector.collectable.directories import SimpleDirectoryCollectable as DirectoryItem

from xpyv import project_root_path

GETDMPNODE_COMMAND = ['/bin/sh', '-c',"vxdmpadm getdmpnode enclosure=Disk | awk '/Disk/{print $1}' | while read d; do echo $d; vxdmpadm getsubpaths dmpnodename=$d; echo; done"]
HOST_ATTACH_DIR = path.abspath(path.join(project_root_path.find(False)))
HOST_ATTACH_BINDIR = path.join(HOST_ATTACH_DIR, 'bin')
XIV_DEVLIST_PATH = path.join(HOST_ATTACH_BINDIR, 'xiv_devlist')
XIV_FC_ADMIN_PATH = path.join(HOST_ATTACH_BINDIR, 'xiv_fc_admin')
XIV_ISCSI_ADMIN_PATH = path.join(HOST_ATTACH_BINDIR, 'xiv_iscsi_admin')
INQUIRY_PATH = path.join(HOST_ATTACH_DIR, 'lib', 'xiv_diag', 'scripts', 'inquiry.py')

class XIVDiag(IXIVDiag):
    def add_collectables(self):
        self.add_collectable_item(FileItem("vxvm-sysboot",
                                       self.archive, '/sbin/init.d/vxvm-sysboot'))
        self.add_collectable_item(CommandItem('Package list', ['swlist' , '']))
        self.add_collectable_item(CommandItem("vol_list",
                                       ['vxdmpadm', 'getdmpnode', 'enclosure=Disk']))
        self.add_collectable_item(CommandItem("multipath", GETDMPNODE_COMMAND))
        self.add_collectable_item(CommandItem('xiv_devlist' , [XIV_DEVLIST_PATH, '-o', 'all', '-d']))
        self.add_collectable_item(CommandItem('xiv_fc_admin -V' , [XIV_FC_ADMIN_PATH, '-L']))
        self.add_collectable_item(CommandItem('xiv_fc_admin -L' , [XIV_FC_ADMIN_PATH, '-L']))
        self.add_collectable_item(CommandItem('xiv_fc_admin -P' , [XIV_FC_ADMIN_PATH, '-P']))
        self.add_collectable_item(CommandItem('xiv_iscsi_admin -V' , [XIV_FC_ADMIN_PATH, '-L']))
        self.add_collectable_item(CommandItem('xiv_iscsi_admin -L' , [XIV_FC_ADMIN_PATH, '-L']))
        self.add_collectable_item(CommandItem('xiv_iscsi_admin -P' , [XIV_ISCSI_ADMIN_PATH, '-P']))
        self.add_collectable_item(CommandItem('inquiry.py', \
            ['xpyv', INQUIRY_PATH, self.collector.tempdir]))
        self.add_collectable_item(FileItem("syslog", self.archive,
                                       '/var/adm/syslog/syslog.log' ))
        self.add_collectable_item(DirectoryItem("/var/adm directory", self.archive,
                                       '/var/adm' ))
        self.add_collectable_item(CommandItem('uname', ['uname' , '-a']))
        self.add_collectable_item(CommandItem('dmesg', ['dmesg' , '']))
 
